#include"oat_master.hpp"
#include"oat_buttonmodule.hpp"
#include"oat.hpp"




namespace oat{




Master::
Master():
current(nullptr),
kbd_callback(nullptr),
kbd_userdata(nullptr),
drop_callback(nullptr),
drop_userdata(nullptr),
mousegrabber(nullptr)
{
}


Master::
~Master()
{
}




void
Master::
set_keyboard_callback(KeyboardCallback  cb, void*  userdata)
{
  kbd_callback = cb;
  kbd_userdata = userdata;
}


void
Master::
set_drop_callback(DropCallback  cb, void*  userdata)
{
  drop_callback = cb;
  drop_userdata = userdata;
}


void
Master::
change_mousegrabber(Widget*  w)
{
  mousegrabber = w;
}


void
Master::
process(Mouse&  mouse)
{
    if(mousegrabber)
    {
      auto&  rect = mousegrabber->get_content_rectangle();

        if(rect.grab(mouse.point) && mouse.change_real_point)
        {
          mouse.change_real_point(mouse.point.x,mouse.point.y);
        }


      auto  w = mousegrabber;

      w->process_mouse(mouse);
      w->ascend_process_mouse(mouse);

      return;
    }


    if(current)
    {
      auto  res = current->scan(mouse.point);

        if(res == current)
        {
          current->process_mouse(mouse);

          current->ascend_process_mouse(mouse);

          return;
        }

      else
        {
          current->process_when_mouse_left();

          current = nullptr;
        }
    }


    if(!current)
    {
      current = scan(mouse.point);

        if(current)
        {
          current->process_when_mouse_entered();

          current->process_mouse(mouse);

          current->ascend_process_mouse(mouse);
        }
    }
}


void
Master::
process(const Keyboard&  kbd)
{
    if(kbd_callback)
    {
      kbd_callback(kbd,kbd_userdata);
    }
}


void
Master::
process(const std::string&  path)
{
    if(drop_callback)
    {
      drop_callback(path,drop_userdata);
    }
}


bool
Master::
update()
{
  ButtonModule::unset_shortlived_all();

    if(test_flag(Flag::needed_to_reform))
    {
      reform();

        if((module.get_width()  < width ) ||
           (module.get_height() < height))
        {
          module.resize(width,height);
        }
    }


  bool  flag = (test_flag(Flag::needed_to_redraw_perfect) ||
                test_flag(Flag::needed_to_redraw_self   ) ||
                test_flag(Flag::needed_to_redraw_child  ));

  try_redraw();

  return flag;
}


Color&
Master::
get_pixel(int  x, int  y) const
{
  return module.get_element(x,y);
}


const DirectColorImageModule&
Master::
get_module() const
{
  return module;
}


}




